// copyMe.controller.ts
import { Request, Response } from "express";
import { db } from "../db";
import { Prisma } from "@prisma/client";

export const getChangeLogUser = async (req: Request, res: Response) => {

  const page = parseInt(req.query.page as string) || 1;
  const pageSize = parseInt(req.query.results as string) || 10;
  const sortField = (req.query.sortField as string) || 'createdAt';
  const sortOrder = (req.query.sortOrder as string) === 'ascend' ? 'asc' : 'desc';
  const search = req.query.search as string || '';

  const filterCondition: Prisma.ChangeLogWhereInput = {
    createdAt: {
      gte: req.query?.createdAtFrom ? new Date(req.query?.createdAtFrom as string) : undefined,
      lte: req.query?.createdAtTo ? new Date(req.query?.createdAtTo as string) : undefined,
    },
    module: req.query.module ? {
      mode: 'insensitive',
      in: Array.isArray(req.query.module) ? req.query.module as [] : [req.query.module as string]
    } : undefined,
    description: req.query.description ? {
      mode: 'insensitive',
      contains: req.query.description as string,
    } : undefined,
    newValue: req.query.newValue ? {
      mode: 'insensitive',
      contains: req.query.newValue as string,
    } : undefined,
    oldValue: req.query.oldValue ? {
      mode: 'insensitive',
      contains: req.query.oldValue as string,
    } : undefined,

  }
  try {
    const data = await db.changeLog.findMany({
      select: {
        id: true,
        description: true,
        module: true,
        newValue: true,
        oldValue: true,
        createdAt: true,
      },
      where: {
        userId: req.user?.userId || "",
        ...filterCondition,
        AND: [
          {
            OR: [
              {
                description: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                module: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                oldValue: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                newValue: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              }
            ]
          }
        ]
      },
      skip: (page - 1) * pageSize,
      take: pageSize,
      orderBy: {
        [sortField]: sortOrder,
      },
    });
    const totalCountData = await db.changeLog.count({
      where: {
        userId: req.user?.userId || "",
        ...filterCondition,
        AND: [
          {
            OR: [
              {
                description: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                module: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                oldValue: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                newValue: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              }
            ]
          }
        ]
      }
    });


    res.status(200).json({
      success: true,
      message: "Successfully",
      data: data,
      total: totalCountData

    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const getChangeLogBusiness = async (req: Request, res: Response) => {
  try {
    const page = parseInt(req.query.page as string) || 1;
    const pageSize = parseInt(req.query.results as string) || 10;
    const sortField = (req.query.sortField as string) || 'createdAt';
    const sortOrder = (req.query.sortOrder as string) === 'ascend' ? 'asc' : 'desc';
    const search = req.query.search as string || '';

    const filterCondition: Prisma.ChangeLogWhereInput = {
      createdAt: {
        gte: req.query?.createdAtFrom ? new Date(req.query?.createdAtFrom as string) : undefined,
        lte: req.query?.createdAtTo ? new Date(req.query?.createdAtTo as string) : undefined,
      },
      module: req.query.module ? {
        mode: 'insensitive',
        in: Array.isArray(req.query.module) ? req.query.module as [] : [req.query.module as string]
      } : undefined,
      description: req.query.description ? {
        mode: 'insensitive',
        contains: req.query.description as string,
      } : undefined,
      newValue: req.query.newValue ? {
        mode: 'insensitive',
        contains: req.query.newValue as string,
      } : undefined,
      oldValue: req.query.oldValue ? {
        mode: 'insensitive',
        contains: req.query.oldValue as string,
      } : undefined,
    }

    const data = await db.changeLog.findMany({
      select: {
        description: true,
        id: true,
        module: true,
        newValues: true,
        oldValues: true,
        createdAt: true,
      },
      where: {
        role: {
          in: ["ADMIN"],
        },
        ...filterCondition,
        AND: [
          {
            OR: [
              {
                description: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                module: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                oldValue: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                newValue: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              }
            ]
          }
        ]
      },
      skip: (page - 1) * pageSize,
      take: pageSize,
      orderBy: {
        [sortField]: sortOrder,
      },
    });


    const totalCountData = await db.changeLog.count({
      where: {
        role: {
          in: ["ADMIN"],
        },
        ...filterCondition,
        AND: [
          {
            OR: [
              {
                description: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              },
              {
                module: {
                  contains: search,
                  mode: 'insensitive', // Case insensitive search
                }
              }
            ]
          }
        ]
      }
    });
    res.status(200).json({
      success: true,
      data: data,
      total: totalCountData
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};
