// copyMe.controller.ts
import { Request, Response } from "express";
import { db } from "../db";
import { addChangeLog } from "../helpers/changeLog.helper";
import bcrypt from "bcrypt";

export const getUserById = async (req: Request, res: Response) => {
  try {
    const { id} = req.params;
    const { businessInfo,businessInfoAdvertisement} = req.query;
    const findResult = await db.users.findUnique({
      select: {
        id: true,
        firstName: true,
        lastName: true,
        middleName: true,
        email: true,
        phoneNumber: true,
        country: true,
        designation: true,
        username: true,
        gender: true,
        imageUrl: true,
        dob: true,
        role: true,
        lastLoginAt:true,
      },
      where: {
        id: id,
      },
    });
    res.status(200).json({
      success: true,
      message: "Successfully",
      data: findResult,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const getUsers = async (req: Request, res: Response) => {
  try {
    const findResult = await db.users.findMany({
      select: {
        id: true,
        firstName: true,
        lastName: true,
        middleName: true,
        email: true,
        phoneNumber: true,
        country: true,
        username: true,
        gender: true,
        imageUrl: true,
        dob: true,
        role: true,
        devices: {
          take: 1,
          orderBy: {
            createdAt: 'desc',  // Adjust the field name if necessary
          },
        },
      },
    });
    res.status(200).json({
      success: true,
      message: "Successfully",
      data: findResult,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};

export const updateUser = async (req: Request, res: Response) => {
  try {
    const { gender, dob, firstName, lastName, middleName, imageUrl, designation } = req.body;

    let error = false;
    let errorDetails = [];

    const findResult = await db.users.findFirst({
      where: {
        id: req.user?.userId,
      },
    });

    if (!findResult) {
      error = true;
      errorDetails.push({
        path: "user",
        message: "User not found",
      });
    }

    if (error) {
      return res.status(400).json({
        success: false,
        message: "Invalid data",
        details: errorDetails,
      });
    }

    const saveData = await db.users.update({
      where: {
        id: req.user?.userId,
      },
      data: {
        firstName,
        lastName,
        middleName,
        designation,
        gender,
        imageUrl,
        dob: dob && new Date(dob),
      },
    });

    addChangeLog({
      userId: req.user?.userId || "",
      role: req.user?.role || "DEFAULT",
      module: "user",
      description: `Update User : ${findResult?.username || ""}`,
      oldValues: findResult || {},
      newValues: saveData,
    });

    res.status(200).json({
      success: true,
      message: "Updated successfully",
      data: saveData,
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};



export const changePassword = async (req: Request, res: Response) => {
  try {
    const { oldPassword, newPassword } = req.body;

    const user = await db.users.findUnique({
      where: { id: req.user?.userId },
    });
    if (!user) {
      return res
        .status(401)
        .json({ success: false, message: "Username not found", details: { path: "username" } });
    }
    if (!(await bcrypt.compare(oldPassword, user.password))) {
      return res
        .status(401)
        .send({ success: false, message: "Invalid Old Password", details: { path: "oldPassword" } });
    }

    const hashedPassword = await bcrypt.hash(newPassword, 10);

    const findResult = await db.users.update({
      where: {
        id: user.id,
      },
      data: {
        password: hashedPassword
      }
    });

    addChangeLog({
      userId: req.user?.userId || "",
      role: req.user?.role || "DEFAULT",
      module: "user",
      description: `Profile Password Updated`,
      oldValues: user || {},
      newValues: findResult,
    });

    res.status(200).json({
      success: true,
      message: "Password Updated successfully",
    });
  } catch (error) {
    console.error(error);
    res.status(500).json({ message: "Internal Server Error", success: false });
  }
};